

#include "CToneHeader.h"




CToneHeader::CToneHeader()
{
	type = 0;	

	address = 0;
	loop = 0;
	end = 0;

	lfo = 0;
	ar = 0;	
	dl = 0;	
	rr = 0;	
	am = 0;	

	size = 0;

	msb = 0;
}


CToneHeader::~CToneHeader()
{

}


void CToneHeader::SetType(unsigned char t)
{
	type = t;
}


unsigned char CToneHeader::GetType()
{
	return type;
}


unsigned short CToneHeader::GetSize()
{
	return size;
}


bool CToneHeader::operator==(CToneHeader &th)
{
	if (th.type != type)
		return false;

	if (th.type & 32)
	{	//rom tone
		if (th.address != address)
			return false;

		if (th.msb != msb)
			return false;
	}
	else
	{	//new sample
		if (th.size != size)
			return false;
	}

	if (th.am != am)
		return false;

	if (th.ar != ar)
		return false;

	if (th.dl != dl)
		return false;

	if (th.end != end)
		return false;

	if (th.lfo != lfo)
		return false;

	if (th.loop != loop)
		return false;

	if (th.rr != rr)
		return false;

	return true;
}


bool CToneHeader::operator!=(CToneHeader &th)
{
	return !(operator==(th));
}


CToneHeader& CToneHeader::operator=(CToneHeader &th)
{
	type = th.type;

	if (th.type & 32)
	{
		address = th.address;
		msb = th.msb;
	}
	else
		size = th.size;

	am = th.am;
	ar = th.ar;
	dl = th.dl;
	end = th.end;
	lfo = th.lfo;
	loop = th.loop;
	rr = th.rr;
	
	return *this;
}



int CToneHeader::Read(istream &in)
{
	//read LSB tone address, just store it here even if we're not dealing with rom-tones
	in.read((char*)&address, 2);

	if ( !(type & 32) )
		address = 0;

	//read loop point
	in.read((char*)&loop, 2);

	//read end point
	in.read((char*)&end, 2);

	//read lfo/vib settings
	in.read(&lfo, 1);

	//read ar/d1r settings
	in.read(&ar, 1);

	//read dl/d2r settings
	in.read(&dl, 1);

	//read rr
	in.read(&rr, 1);

	//read am
	in.read(&am, 1);

	//check tone type
	if (type & 32)
	{	//rom tone
		in.seekg(1, ios::cur);

		in.read(&msb, 1);

		size = 0;
	}
	else
	{	//new sample
		//read size
		in.read((char*)&size, 2);
	}

	return 0;
}


int CToneHeader::Write(ostream &out)
{
	//output LSB starting address (rom tone) OR zero's (new sample)
	out.write((char*)&address, 2);

	out.write((char*)&loop, 2);

	out.write((char*)&end, 2);

	out.write(&lfo, 1);

	out.write(&ar, 1);

	out.write(&dl, 1);

	out.write(&rr, 1);

	out.write(&am, 1);

	if (type & 32)
	{
		//output 0
		out.write((char*)&size, 1);

		out.write(&msb, 1);
	}
	else
		out.write((char*)&size, 2);

	return 0;
}


ostream& operator<<(ostream &out, CToneHeader &th)
{
	//output type
	out.write(&th.type, 1);

	//check tone type
	if (th.type & 32)
	{	//rom tone
		//output MSB tone-address
		out.write(&th.msb, 1);

		//output LSB tone-address 
		out.write((char*)&th.address, 2);
	}
	else
	{	//own sample
		//output filler byte
		out.write((char*)&th.size, 1);

		//output sample size
		out.write((char*)&th.size, 2);
	}

	//output loop point
	out.write((char*)&th.loop, 2);

	//output end point
	out.write((char*)&th.end, 2);

	//output lfo/vib settings
	out.write(&th.lfo, 1);

	//output ar/d1r settings
	out.write(&th.ar, 1);

	//output dl/d2r settings
	out.write(&th.dl, 1);

	//output rr
	out.write(&th.rr, 1);

	//output am
	out.write(&th.am, 1);

	return out;
}


CToneHeader& operator>>(istream &in, CToneHeader &th)
{
	//read type
	in.read(&th.type, 1);

	//check tone type
	if (th.type & 32)
	{	//rom tone
		//read MSB tone address
		in.read(&th.msb, 1);

		//read LSB tone address
		in.read((char*)&th.address, 2);
	}
	else
	{	//new sample
		//read filler byte
		in.read((char*)&th.size, 1);

		//read size
		in.read((char*)&th.size, 2);
	}

	//read loop point
	in.read((char*)&th.loop, 2);

	//read end point
	in.read((char*)&th.end, 2);

	//read lfo/vib settings
	in.read(&th.lfo, 1);

	//read ar/d1r settings
	in.read(&th.ar, 1);

	//read dl/d2r settings
	in.read(&th.dl, 1);

	//read rr
	in.read(&th.rr, 1);

	//read am
	in.read(&th.am, 1);

	return th;
}
